@@ -15,11 +15,10 @@ import logging
 from tryton import version
 from tryton import config
 from tryton.config import CONFIG, CURRENT_DIR, PREFIX, PIXMAPS_DIR, \
-        TRYTON_ICON, get_home_dir
+        TRYTON_ICON, get_config_dir
 from tryton import translate
 from tryton import gui
 import traceback
-import mx.DateTime
 import time
 import signal
 
@@ -28,6 +27,7 @@ class TrytonClient(object):
     "Tryton client"
 
     def __init__(self):
+        CONFIG.parse()
         logging.basicConfig()
         translate.set_language_direction(CONFIG['client.language_direction'])
         translate.setlang(CONFIG['client.lang'])
@@ -46,10 +46,6 @@ class TrytonClient(object):
             logging.getLogger().setLevel(
                     loglevel[CONFIG['logging.default'].upper()])
 
-        if not hasattr(mx.DateTime, 'strptime'):
-            mx.DateTime.strptime = lambda x, y: mx.DateTime.mktime(
-                    time.strptime(x, y))
-
         factory = gtk.IconFactory()
         factory.add_default()
 
@@ -105,7 +101,7 @@ class TrytonClient(object):
         except KeyboardInterrupt:
             CONFIG.save()
             if hasattr(gtk, 'accel_map_save'):
-                gtk.accel_map_save(os.path.join(get_home_dir(), '.trytonsc'))
+                gtk.accel_map_save(os.path.join(get_config_dir(), 'accel.map'))
 
 if __name__ == "__main__":
     CLIENT = TrytonClient()
