@@ -16,13 +16,13 @@ class CellRendererFloat(CellRendererInteger):
         position = entry.get_position()
         new_value = value[:position] + new_text + value[position:]
         try:
-            if new_value == '-':
+            decimal_point = locale.localeconv()['decimal_point']
+
+            if new_value in ('-', decimal_point):
                 return
 
             locale.atof(new_value)
 
-            decimal_point = locale.localeconv()['decimal_point']
-
             new_int = new_value
             new_decimal = ''
             if decimal_point in new_value:
