@@ -14,33 +14,18 @@ import gtk
 _ = gettext.gettext
 
 def get_home_dir():
-    """
-    Return the closest possible equivalent to a 'home' directory.
-    For Posix systems, this is $HOME, and on NT it's $HOMEDRIVE\$HOMEPATH.
-    Currently only Posix and NT are implemented, a HomeDirError exception is
-    raised for all other OSes.
-    """
+    if os.name == 'nt':
+        return os.path.join(os.environ['HOMEDRIVE'], os.environ['HOMEPATH'])
+    return os.environ['HOME']
 
-    if os.name == 'posix':
-        return os.path.expanduser('~')
-    elif os.name == 'nt':
-        try:
-            return os.path.join(os.environ['HOMEDRIVE'], os.environ['HOMEPATH'])
-        except:
-            try:
-                import _winreg as wreg
-                key = wreg.OpenKey(wreg.HKEY_CURRENT_USER,
-                        "Software\Microsoft\Windows\Current" \
-                                "Version\Explorer\Shell Folders")
-                homedir = wreg.QueryValueEx(key, 'Personal')[0]
-                key.Close()
-                return homedir
-            except:
-                return 'C:\\'
-    elif os.name == 'dos':
-        return 'C:\\'
-    else:
-        return '.'
+def get_config_dir():
+    if os.name == 'nt':
+        return os.path.join(os.environ['APPDATA'], '.config', 'tryton',
+                VERSION.rsplit('.', 1)[0])
+    return os.path.join(os.environ['HOME'], '.config', 'tryton',
+            VERSION.rsplit('.', 1)[0])
+if not os.path.isdir(get_config_dir()):
+    os.makedirs(get_config_dir(), 0700)
 
 def find_path(progs, args):
     if os.name == 'nt':
@@ -104,6 +89,8 @@ class ConfigManager(object):
         self.options = {
             'login.host': True
         }
+
+    def parse(self):
         parser = optparse.OptionParser(version=("Tryton %s" % VERSION))
         parser.add_option("-c", "--config", dest="config",
                 help=_("specify alternate config file"))
@@ -126,7 +113,7 @@ class ConfigManager(object):
 
         if opt.config and not os.path.isfile(opt.config):
             raise Exception(_('File "%s" not found') % (opt.config,))
-        self.rcfile = opt.config or os.path.join(get_home_dir(), '.tryton')
+        self.rcfile = opt.config or os.path.join(get_config_dir(), 'tryton.conf')
         self.load()
 
         for arg in ('log_level', 'log_logger'):
@@ -149,7 +136,7 @@ class ConfigManager(object):
                 if not configparser.has_section(section):
                     configparser.add_section(section)
                 configparser.set(section, name, self.config[entry])
-            configparser.write(file(self.rcfile, 'wb'))
+            configparser.write(open(self.rcfile, 'wb'))
         except:
             logging.getLogger('common.options').warn(
                     _('Unable to write config file %s!') % \
@@ -192,7 +179,7 @@ if not os.path.isdir(PIXMAPS_DIR):
         PIXMAPS_DIR = os.path.join(PREFIX, 'share', 'pixmaps', 'tryton')
 
 TRYTON_ICON = gtk.gdk.pixbuf_new_from_file(
-        os.path.join(PIXMAPS_DIR, '..', 'tryton-icon.png'))
+        os.path.join(PIXMAPS_DIR, 'tryton-icon.png'))
 
 def _data_dir():
     data_dir = os.path.join(CURRENT_DIR, 'share', 'tryton')
