@@ -7,7 +7,8 @@ import urlparse
 import gobject
 import gtk
 import tryton.rpc as rpc
-from tryton.config import CONFIG, TRYTON_ICON, PIXMAPS_DIR, DATA_DIR, get_home_dir
+from tryton.config import CONFIG, TRYTON_ICON, PIXMAPS_DIR, DATA_DIR, \
+        get_config_dir
 import tryton.common as common
 from tryton.action import Action
 from tryton.gui.window import Window
@@ -31,6 +32,10 @@ try:
     import igemacintegration
 except ImportError:
     igemacintegration = None
+try:
+    import gtkspell
+except:
+    gtkspell = None
 
 _ = gettext.gettext
 
@@ -97,7 +102,7 @@ class Main(object):
                 gtk.gdk.CONTROL_MASK)
 
         if hasattr(gtk, 'accel_map_load'):
-            gtk.accel_map_load(os.path.join(get_home_dir(), '.trytonsc'))
+            gtk.accel_map_load(os.path.join(get_config_dir(), 'accel.map'))
 
         self.tooltips = common.Tooltips()
 
@@ -677,15 +682,16 @@ class Main(object):
         if CONFIG['client.save_width_height']:
             checkmenuitem_save_width_height.set_active(True)
 
-        checkmenuitem_spellcheck = gtk.CheckMenuItem(_('Spell Checking'))
-        checkmenuitem_spellcheck.connect('activate',
-                lambda menuitem: CONFIG.__setitem__('client.spellcheck',
-                    menuitem.get_active()))
-        checkmenuitem_spellcheck.set_accel_path(
-                '<tryton>/Options/Form/Spell Checking')
-        menu_form.add(checkmenuitem_spellcheck)
-        if CONFIG['client.spellcheck']:
-            checkmenuitem_spellcheck.set_active(True)
+        if gtkspell:
+            checkmenuitem_spellcheck = gtk.CheckMenuItem(_('Spell Checking'))
+            checkmenuitem_spellcheck.connect('activate',
+                    lambda menuitem: CONFIG.__setitem__('client.spellcheck',
+                        menuitem.get_active()))
+            checkmenuitem_spellcheck.set_accel_path(
+                    '<tryton>/Options/Form/Spell Checking')
+            menu_form.add(checkmenuitem_spellcheck)
+            if CONFIG['client.spellcheck']:
+                checkmenuitem_spellcheck.set_active(True)
 
         menuitem_tab = gtk.MenuItem(_('Tabs Position'))
         menu_form.add(menuitem_tab)
@@ -1270,9 +1276,8 @@ class Main(object):
         CONFIG['client.default_height'] = Main.get_main()._height
         CONFIG.save()
         if hasattr(gtk, 'accel_map_save'):
-            gtk.accel_map_save(os.path.join(get_home_dir(), '.trytonsc'))
+            gtk.accel_map_save(os.path.join(get_config_dir(), 'accel.map'))
         gtk.main_quit()
-        sys.exit()
 
     def sig_close(self, widget):
         if common.sur(_("Do you really want to quit?"), parent=self.window):
@@ -1352,7 +1357,10 @@ class Main(object):
             if self.buttons[i]:
                 self.buttons[i].set_sensitive(
                         bool(view and (i in view.handlers)))
-        self.buttons['but_attach'].set_stock_id('tryton-attachment')
+        if hasattr(view, 'update_attachment_count'):
+            view.update_attachment_count()
+        else:
+            self._attachment_count(view, 0)
 
     def _attachment_count(self, widget, signal_data):
         label = _('Attachment(%d)') % signal_data
@@ -1488,7 +1496,7 @@ class Main(object):
         dialog = DBRestore(self.window, filename=filename)
         url, dbname, passwd, update = dialog.run(self.window)
         if dbname:
-            file_p = file(filename, 'rb')
+            file_p = open(filename, 'rb')
             data_b64 = base64.encodestring(file_p.read())
             file_p.close()
             host, port = url.rsplit(':' , 1)
@@ -1567,7 +1575,7 @@ class Main(object):
                 filename=dbname + '-' + time.strftime('%Y%m%d%H%M') + '.dump')
 
         if filename:
-            file_ = file(filename, 'wb')
+            file_ = open(filename, 'wb')
             file_.write(dump)
             file_.close()
             common.message(_("Database backuped successfully!"), \
