@@ -10,6 +10,10 @@ from tryton.gui.window.win_search import WinSearch
 from tryton.action import Action as Action2
 from tryton.gui.window.view_tree.view_tree import ViewTree
 import tryton.common as common
+from tryton.pyson import PYSONDecoder
+import gettext
+_LIMIT = 2000
+_ = gettext.gettext
 
 
 class Action(object):
@@ -27,6 +31,8 @@ class Action(object):
             common.process_exception(exception, self._window)
             raise
 
+        view_ids = False
+        self.action['view_mode'] = None
         if self.action.get('views', []):
             view_ids = [x[0] for x in self.action['views']]
             self.action['view_mode'] = [x[1] for x in self.action['views']]
@@ -36,99 +42,103 @@ class Action(object):
         if 'view_mode' in attrs:
             self.action['view_mode'] = attrs['view_mode']
 
-        self.action.setdefault('domain', '[]')
+        self.action.setdefault('pyson_domain', '[]')
         self.context = {'active_id': False, 'active_ids': []}
-        self.context.update(common.safe_eval(self.action.get('context', '{}'),
-            self.context.copy()))
+        self.context.update(PYSONDecoder(self.context).decode(
+            self.action.get('pyson_context', '{}')))
 
         eval_ctx = self.context.copy()
-        eval_ctx['datetime'] = datetime
-        self.context.update(common.safe_eval(self.action.get('context', '{}'),
-            eval_ctx))
+        self.context.update(PYSONDecoder(eval_ctx).decode(
+            self.action.get('pyson_context', '{}')))
 
         domain_ctx = self.context.copy()
-        domain_ctx['time'] = time
-        domain_ctx['datetime'] = datetime
-        self.domain = common.safe_eval(self.action['domain'] or '[]',
-                domain_ctx)
+        self.domain = PYSONDecoder(domain_ctx).decode(
+                self.action['pyson_domain'])
 
 
         self.widget = gtk.Frame()
-        self.widget.set_border_width(5)
+        self.widget.set_border_width(0)
 
-        hbox = gtk.HBox(homogeneous=False, spacing=5)
-        self.widget.set_label_widget(hbox)
-        self.widget.set_label_align(1, 0.5)
+        vbox = gtk.VBox(homogeneous=False, spacing=0)
+        hbox = gtk.HBox(homogeneous=False, spacing=0)
+        alignment = gtk.Alignment(1.0)
+        alignment.set_padding(0, 0, 0, 0)
+        alignment.add(hbox)
+        vbox.pack_start(alignment, expand=False, fill=True)
+        self.widget.add(vbox)
 
         self.title = gtk.Label()
-        hbox.pack_start(self.title, expand=True, fill=True)
+        self.widget.set_label_widget(self.title)
+        self.widget.set_label_align(0.0, 0.5)
 
         tooltips = common.Tooltips()
 
-        hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
-
         if self.action['view_type'] == 'form':
-            eb_search = gtk.EventBox()
-            tooltips.set_tip(eb_search, _('Search'))
-            eb_search.set_events(gtk.gdk.BUTTON_PRESS)
-            eb_search.connect('button_press_event', self._sig_search)
+            but_search = gtk.Button()
+            tooltips.set_tip(but_search, _('Search'))
+            but_search.connect('clicked', self._sig_search)
             img_search = gtk.Image()
-            img_search.set_from_stock('tryton-find', gtk.ICON_SIZE_BUTTON)
+            img_search.set_from_stock('tryton-find',
+                    gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_search.set_alignment(0.5, 0.5)
-            eb_search.add(img_search)
-            hbox.pack_start(eb_search, expand=False, fill=False)
+            but_search.add(img_search)
+            but_search.set_relief(gtk.RELIEF_NONE)
+            hbox.pack_start(but_search, expand=False, fill=False)
 
-        eb_open = gtk.EventBox()
-        tooltips.set_tip(eb_open, _('Open'))
-        eb_open.set_events(gtk.gdk.BUTTON_PRESS)
-        eb_open.connect('button_press_event', self._sig_open)
+        but_open = gtk.Button()
+        tooltips.set_tip(but_open, _('Open'))
+        but_open.connect('clicked', self._sig_open)
         img_open = gtk.Image()
-        img_open.set_from_stock('tryton-open', gtk.ICON_SIZE_BUTTON)
+        img_open.set_from_stock('tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR)
         img_open.set_alignment(0.5, 0.5)
-        eb_open.add(img_open)
-        hbox.pack_start(eb_open, expand=False, fill=False)
+        but_open.add(img_open)
+        but_open.set_relief(gtk.RELIEF_NONE)
+        hbox.pack_start(but_open, expand=False, fill=False)
 
-        hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
 
         if self.action['view_type'] == 'form':
-            eb_previous = gtk.EventBox()
-            tooltips.set_tip(eb_previous, _('Previous'))
-            eb_previous.set_events(gtk.gdk.BUTTON_PRESS)
-            eb_previous.connect('button_press_event', self._sig_previous)
+            hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
+            but_previous = gtk.Button()
+            tooltips.set_tip(but_previous, _('Previous'))
+            but_previous.connect('clicked', self._sig_previous)
             img_previous = gtk.Image()
-            img_previous.set_from_stock('tryton-go-previous', gtk.ICON_SIZE_BUTTON)
+            img_previous.set_from_stock('tryton-go-previous',
+                    gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_previous.set_alignment(0.5, 0.5)
-            eb_previous.add(img_previous)
-            hbox.pack_start(eb_previous, expand=False, fill=False)
+            but_previous.add(img_previous)
+            but_previous.set_relief(gtk.RELIEF_NONE)
+            hbox.pack_start(but_previous, expand=False, fill=False)
 
             self.label = gtk.Label('(0,0)')
             hbox.pack_start(self.label, expand=False, fill=False)
 
-            eb_next = gtk.EventBox()
-            tooltips.set_tip(eb_next, _('Next'))
-            eb_next.set_events(gtk.gdk.BUTTON_PRESS)
-            eb_next.connect('button_press_event', self._sig_next)
+            but_next = gtk.Button()
+            tooltips.set_tip(but_next, _('Next'))
+            but_next.connect('clicked', self._sig_next)
             img_next = gtk.Image()
-            img_next.set_from_stock('tryton-go-next', gtk.ICON_SIZE_BUTTON)
+            img_next.set_from_stock('tryton-go-next',
+                    gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_next.set_alignment(0.5, 0.5)
-            eb_next.add(img_next)
-            hbox.pack_start(eb_next, expand=False, fill=False)
+            but_next.add(img_next)
+            but_next.set_relief(gtk.RELIEF_NONE)
+            hbox.pack_start(but_next, expand=False, fill=False)
 
             hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
 
-            eb_switch = gtk.EventBox()
-            tooltips.set_tip(eb_switch, _('Switch'))
-            eb_switch.set_events(gtk.gdk.BUTTON_PRESS)
-            eb_switch.connect('button_press_event', self._sig_switch)
+            but_switch = gtk.Button()
+            tooltips.set_tip(but_switch, _('Switch'))
+            but_switch.connect('clicked', self._sig_switch)
             img_switch = gtk.Image()
-            img_switch.set_from_stock('tryton-fullscreen', gtk.ICON_SIZE_BUTTON)
+            img_switch.set_from_stock('tryton-fullscreen',
+                    gtk.ICON_SIZE_SMALL_TOOLBAR)
             img_switch.set_alignment(0.5, 0.5)
-            eb_switch.add(img_switch)
-            hbox.pack_start(eb_switch, expand=False, fill=False)
+            but_switch.add(img_switch)
+            but_switch.set_relief(gtk.RELIEF_NONE)
+            hbox.pack_start(but_switch, expand=False, fill=False)
 
         alignment = gtk.Alignment(0.5, 0.5, 1.0, 1.0)
-        alignment.set_padding(8, 0, 0, 0)
-        self.widget.add(alignment)
+        alignment.set_padding(0, 2, 2, 2)
+        vbox.pack_start(alignment, expand=True, fill=True)
 
         self.widget.show_all()
 
@@ -136,9 +146,10 @@ class Action(object):
             self.screen = Screen(self.action['res_model'], self._window,
                     view_type=self.action['view_mode'], context=self.context,
                     view_ids=view_ids, domain=self.domain, readonly=True)
-            alignment.add(self.screen.widget)
-            self.title.set_text(attrs.get('string',
-                self.screen.current_view.title))
+            self.screen.screen_container.alternate_view = True
+            self.screen.switch_view(view_type=self.action['view_mode'])
+            alignment.add(self.screen.screen_container.alternate_viewport)
+            name = self.screen.current_view.title
             self.screen.signal_connect(self, 'record-message', self._sig_label)
         elif self.action['view_type'] == 'tree':
             ctx = {}
@@ -159,45 +170,48 @@ class Action(object):
             self.tree = ViewTree(view, [], self._window, True,
                     context=ctx)
             alignment.add(self.tree.widget_get())
-            self.title.set_text(attrs.get('string',
-                self.tree.name))
+            name = self.tree.name
             self.tree.view.connect('key_press_event', self.sig_key_press)
+
+        if attrs.get('string'):
+            self.title.set_text(attrs['string'])
+        elif self.action.get('window_name'):
+            self.title.set_text(self.action['name'])
+        else:
+            self.title.set_text(name)
+
         self.widget.set_size_request(int(attrs.get('width',-1)),
                 int(attrs.get('height', -1)))
         self.display()
 
-    def _sig_switch(self, *args):
+    def _sig_switch(self, widget):
         self.screen.switch_view()
 
-    def _sig_search(self, widget, event):
-        if event.type == gtk.gdk.BUTTON_PRESS:
-            ctx = {}
-            ctx.update(rpc.CONTEXT)
-            ctx.update(self.context)
-            win = WinSearch(self.action['res_model'], domain=self.domain,
-                    context=ctx, parent=self._window)
-            res = win.run()
-            if res:
-                self.screen.clear()
-                self.screen.load(res)
+    def _sig_search(self, widget):
+        ctx = {}
+        ctx.update(rpc.CONTEXT)
+        ctx.update(self.context)
+        win = WinSearch(self.action['res_model'], domain=self.domain,
+                context=ctx, parent=self._window)
+        res = win.run()
+        if res:
+            self.screen.clear()
+            self.screen.load(res)
 
-    def _sig_open(self, widget, event):
-        if event.type == gtk.gdk.BUTTON_PRESS:
-            try:
-                action_id = rpc.execute('model', 'ir.action',
-                        'get_action_id', self.act_id, rpc.CONTEXT)
-            except Exception, exception:
-                common.process_exception(exception, self._window)
-            if action_id:
-                Action2.execute(action_id, {})
+    def _sig_open(self, widget):
+        try:
+            action_id = rpc.execute('model', 'ir.action',
+                    'get_action_id', self.act_id, rpc.CONTEXT)
+        except Exception, exception:
+            common.process_exception(exception, self._window)
+        if action_id:
+            Action2.execute(action_id, {}, self._window)
 
-    def _sig_previous(self, widget, event):
-        if event.type == gtk.gdk.BUTTON_PRESS:
-            self.screen.display_prev()
+    def _sig_previous(self, widget):
+        self.screen.display_prev()
 
-    def _sig_next(self, widget, event):
-        if event.type == gtk.gdk.BUTTON_PRESS:
-            self.screen.display_next()
+    def _sig_next(self, widget):
+        self.screen.display_next()
 
     def _sig_label(self, screen, signal_data):
         name = '_'
@@ -213,7 +227,8 @@ class Action(object):
     def display(self):
         try:
             res_ids = rpc.execute('model', self.action['res_model'], 'search',
-                    self.domain, 0, self.action.get('limit', 80))
+                    self.domain, 0, self.action['limit'] or _LIMIT, None,
+                    rpc.CONTEXT)
         except Exception, exception:
             common.process_exception(exception, self._window)
             return False
