@@ -76,6 +76,9 @@ class CharField(object):
         return self.get(record, check_load=check_load, readonly=True,
                 modified=False)
 
+    def get_on_change_value(self, record, check_load=True):
+        return self.get_eval(record, check_load=check_load)
+
     def set_client(self, record, value, force_change=False):
         internal = record.value.get(self.name, False)
         prev_modified = record.modified
@@ -369,8 +372,7 @@ class O2MField(CharField):
                         get_modifiedonly=modified)
                 if record2.is_modified() and values:
                     result.append(('write', record2.id, values))
-                else:
-                    result[0][1].append(record2.id)
+                result[0][1].append(record2.id)
             else:
                 result.append(('create',
                     record2.get(check_load=check_load, get_readonly=readonly)))
@@ -393,6 +395,9 @@ class O2MField(CharField):
         return result
 
     def get_eval(self, record, check_load=True):
+        return [x.id for x in record.value.get(self.name) or []]
+
+    def get_on_change_value(self, record, check_load=True):
         result = []
         if record.value.get(self.name) is None:
             return []
