@@ -3,9 +3,9 @@
 import gtk
 from tryton.gui.window.view_form.screen import Screen
 from interface import WidgetInterface
-from one2many import Dialog
 import tryton.rpc as rpc
 from tryton.gui.window.win_search import WinSearch
+from tryton.gui.window.win_form import WinForm
 from tryton.gui.window.view_form.widget_search.form import _LIMIT
 import tryton.common as common
 import gettext
@@ -15,8 +15,9 @@ _ = gettext.gettext
 
 class Many2Many(WidgetInterface):
 
-    def __init__(self, window, parent, model, attrs=None):
-        super(Many2Many, self).__init__(window, parent, model, attrs)
+    def __init__(self, field_name, model_name, window, attrs=None):
+        super(Many2Many, self).__init__(field_name, model_name, window,
+                attrs=attrs)
 
         self.widget = gtk.VBox(homogeneous=False, spacing=5)
 
@@ -57,54 +58,74 @@ class Many2Many(WidgetInterface):
 
         self.widget.pack_start(hbox, expand=False, fill=False)
 
-        self.screen = Screen(attrs['relation'], self._window,
+        hbox.set_focus_chain([self.wid_text])
+
+        self.screen = Screen(attrs['relation'], self.window,
                 view_type=['tree'], views_preload=attrs.get('views', {}),
                 row_activate=self._on_activate)
 
         self.widget.pack_start(self.screen.widget, expand=True, fill=True)
 
-        self.old = None
+        self.screen.widget.connect('key_press_event', self.on_keypress)
+        self.wid_text.connect('key_press_event', self.on_keypress)
+
+    def _color_widget(self):
+        if hasattr(self.screen.current_view, 'widget_tree'):
+            return self.screen.current_view.widget_tree
+        return super(Many2Many, self)._color_widget()
 
     def grab_focus(self):
         return self.wid_text.grab_focus()
 
+    def on_keypress(self, widget, event):
+        if event.keyval == gtk.keysyms.F3:
+            self._sig_add()
+            return False
+        if event.keyval == gtk.keysyms.F2 \
+                and widget == self.screen.widget:
+            self._sig_edit()
+        if event.keyval in (gtk.keysyms.Delete, gtk.keysyms.KP_Delete) \
+                and widget == self.screen.widget:
+            self._sig_remove()
+            return False
+
     def destroy(self):
         self.screen.destroy()
         self.widget.destroy()
         del self.widget
 
     def _sig_add(self, *args):
-        domain = self._view.modelfield.domain_get(self._view.model)
-        context = self._view.modelfield.context_get(self._view.model)
+        domain = self.field.domain_get(self.record)
+        context = self.field.context_get(self.record)
         value = self.wid_text.get_text()
 
         try:
+            if value:
+                dom = [('rec_name', 'ilike', '%' + value + '%'), domain]
+            else:
+                dom = domain
             ids = rpc.execute('model', self.attrs['relation'], 'search',
-                    [('rec_name', 'ilike', value), domain], 0, _LIMIT,
-                    None, context)
+                    dom , 0, _LIMIT, None, context)
         except Exception, exception:
-            common.process_exception(exception, self._window)
+            common.process_exception(exception, self.window)
             return False
         if len(ids) != 1 or not value:
             win = WinSearch(self.attrs['relation'], sel_multi=True, ids=ids,
-                    context=context, domain=domain, parent=self._window,
+                    context=context, domain=domain, parent=self.window,
                     views_preload=self.attrs.get('views', {}))
             ids = win.run()
 
         res_id = None
         if ids:
             res_id = ids[0]
-        self.screen.load(ids)
+        self.screen.load(ids, modified=True)
         self.screen.display(res_id=res_id)
         if self.screen.current_view:
             self.screen.current_view.set_cursor()
         self.wid_text.set_text('')
-        self.set_value(self._view.model, self._view.modelfield)
 
     def _sig_remove(self, *args):
-        self.screen.remove()
-        self.screen.display()
-        self.set_value(self._view.model, self._view.modelfield)
+        self.screen.remove(remove=True)
 
     def _sig_activate(self, *args):
         self._sig_add()
@@ -114,20 +135,11 @@ class Many2Many(WidgetInterface):
         self._sig_edit()
 
     def _sig_edit(self):
-        if self.screen.current_model:
-            readonly = False
-            domain = []
-            if self._view.modelfield and self._view.model:
-                modelfield = self._view.modelfield
-                model = self._view.model
-                readonly = modelfield.get_state_attrs(model
-                        ).get('readonly', False)
-                domain = modelfield.domain_get(self._view.model)
-            dia = Dialog(self.attrs['relation'], parent=self._view.model,
-                    model=self.screen.current_model, attrs=self.attrs,
-                    window=self._window, readonly=readonly, domain=domain)
-            res, value = dia.run()
-            dia.destroy()
+        if self.screen.current_record:
+            win = WinForm(self.screen, self.window)
+            if win.run():
+                self.screen.current_record.save()
+            win.destroy()
 
     def _readonly_set(self, value):
         super(Many2Many, self)._readonly_set(value)
@@ -136,20 +148,27 @@ class Many2Many(WidgetInterface):
         self.wid_but_remove.set_sensitive(not value)
         self.wid_but_add.set_sensitive(not value)
 
-    def display(self, model, model_field):
-        super(Many2Many, self).display(model, model_field)
-        ids = []
-        if model_field:
-            ids = model_field.get_client(model)
-        if ids != self.old:
-            self.screen.clear()
-            self.screen.load(ids, set_cursor=False)
-            self.old = ids
+    def display(self, record, field):
+        super(Many2Many, self).display(record, field)
+        if field is None:
+            self.screen.current_record = None
+            self.screen.display()
+            return False
+        new_group = field.get_client(record)
+        if id(self.screen.group) != id(new_group):
+            self.screen.group = new_group
         self.screen.display()
         return True
 
     def display_value(self):
-        return self._view.modelfield.rec_name(self._view.model)
+        ids = self.field.get_default(self.record)
+        try:
+            result = rpc.execute('model', self.attrs['relation'], 'read',
+                    ids, ['rec_name'], rpc.CONTEXT)
+        except:
+            return str(ids)
+        return ', '.join(x['rec_name'] for x in result)
 
-    def set_value(self, model, model_field):
-        model_field.set_client(model, [x.id for x in self.screen.models.models])
+    def set_value(self, record, field):
+        self.screen.current_view.set_value()
+        return True
