@@ -3,12 +3,13 @@
 #This code is inspired by the pycha project (http://www.lorenzogil.com/projects/pycha/)
 import gtk
 from tryton.common import hex2rgb, generateColorscheme, DT_FORMAT, \
-        COLOR_SCHEMES, safe_eval
+        DHM_FORMAT, COLOR_SCHEMES, datetime_strftime
+from tryton.pyson import PYSONDecoder
 import locale
 import math
 import datetime
-import mx.DateTime
 import time
+from dateutil.relativedelta import relativedelta
 import tryton.rpc as rpc
 import cairo
 from tryton.action import Action
@@ -131,7 +132,7 @@ class Graph(gtk.DrawingArea):
     def action_keyword(self, ids, window):
         if not ids:
             return
-        ctx = self.models.context.copy()
+        ctx = self.group.context.copy()
         if 'active_ids' in ctx:
             del ctx['active_ids']
         if 'active_id' in ctx:
@@ -165,7 +166,7 @@ class Graph(gtk.DrawingArea):
             base = 1
         else:
             base = 10**int(math.log(self.yrange, 10))
-        for i in range(int(self.yrange / base) + 1):
+        for i in xrange(int(self.yrange / base) + 1):
             val = int(self.minyval / base) * base + i * base
             h = (val - self.minyval) * self.yscale
             label = locale.format('%.2f', val, True)
@@ -320,21 +321,21 @@ class Graph(gtk.DrawingArea):
         return self.area.x + self.area.w * 0.05, \
                 self.area.y + self.area.h * 0.05
 
-    def display(self, models):
-        self.updateDatas(models)
+    def display(self, group):
+        self.updateDatas(group)
         self.setColorScheme()
         self.updateXY()
         self.updateGraph()
         self.queue_draw()
 
-    def updateDatas(self, models):
+    def updateDatas(self, group):
         self.datas = {}
         self.labels = {}
         self.ids = {}
-        self.models = models
+        self.group = group
         minx = None
         maxx = None
-        for model in models:
+        for model in group:
             x = model[self.xfield['name']].get(model)
             if not minx:
                 minx = x
@@ -353,9 +354,9 @@ class Graph(gtk.DrawingArea):
                 if yfield.get('domain'):
                     values = rpc.CONTEXT.copy()
                     values['state'] = 'draft'
-                    for field in model.mgroup.fields:
+                    for field in model.group.fields:
                         values[field] = model[field].get(model, check_load=False)
-                    if not safe_eval(yfield['domain'], values):
+                    if not PYSONDecoder(values).decode(yfield['domain']):
                         continue
                 if yfield['name'] == '#':
                     self.datas[x][key] += 1
@@ -363,27 +364,23 @@ class Graph(gtk.DrawingArea):
                     self.datas[x][key] += \
                             float(model[yfield['name']].get(model))
         if isinstance(minx, datetime.datetime):
-            date = mx.DateTime.mktime(time.strptime(str(minx), '%Y-%m-%d %H:%M:%S'))
-            end_date = mx.DateTime.mktime(time.strptime(str(maxx), '%Y-%m-%d %H:%M:%S'))
-            while date <= end_date:
-                key = datetime.datetime(date.year, date.month, date.day, 0, 0, 0)
-                self.labels[key] = date.strftime(DT_FORMAT)
-                self.datas.setdefault(key, {})
+            date = minx
+            while date <= maxx:
+                self.labels[date] = datetime_strftime(date, DT_FORMAT)
+                self.datas.setdefault(date, {})
                 for yfield in self.yfields:
-                    self.datas[key].setdefault(
+                    self.datas[date].setdefault(
                             yfield.get('key', yfield['name']), 0.0)
-                date = date + mx.DateTime.RelativeDateTime(days=1)
+                date += relativedelta(days=1)
         elif isinstance(minx, datetime.date):
-            date = mx.DateTime.mktime(time.strptime(str(minx), '%Y-%m-%d'))
-            end_date = mx.DateTime.mktime(time.strptime(str(maxx), '%Y-%m-%d'))
-            while date <= end_date:
-                key = datetime.date(date.year, date.month, date.day)
-                self.labels[key] = date.strftime(DT_FORMAT)
-                self.datas.setdefault(key, {})
+            date = minx
+            while date <= maxx:
+                self.labels[date] = datetime_strftime(date, DT_FORMAT)
+                self.datas.setdefault(date, {})
                 for yfield in self.yfields:
-                    self.datas[key].setdefault(
+                    self.datas[date].setdefault(
                             yfield.get('key', yfield['name']), 0.0)
-                date = date + mx.DateTime.RelativeDateTime(days=1)
+                date += relativedelta(days=1)
 
     def updateArea(self, cr, width, height):
         maxylabel = ''
