@@ -3,7 +3,10 @@
 import pysocket
 import logging
 import socket
+import os
 from threading import Semaphore
+from tryton.fingerprints import Fingerprints
+from tryton.config import get_config_dir
 
 _SOCK = None
 _USER = 0
@@ -15,6 +18,7 @@ _VIEW_CACHE = {}
 TIMEZONE = 'utc'
 SECURE = False
 _SEMAPHORE = Semaphore()
+_CA_CERTS = os.path.join(get_config_dir(), 'ca_certs')
 
 def db_list(host, port):
     global _SOCK, SECURE
@@ -24,7 +28,8 @@ def db_list(host, port):
             if _SOCK and (_SOCK.hostname != host or _SOCK.port != port):
                 _SOCK.disconnect()
             if _SOCK is None:
-                _SOCK = pysocket.PySocket()
+                _SOCK = pysocket.PySocket(fingerprints=Fingerprints(),
+                        ca_certs=_CA_CERTS)
             if not _SOCK.connected:
                 _SOCK.connect(host, port)
             args = (None, None, None, 'common', 'db', 'list')
@@ -55,12 +60,20 @@ def db_exec(host, port, method, *args):
             if _SOCK and (_SOCK.hostname != host or _SOCK.port != port):
                 _SOCK.disconnect()
             if _SOCK is None:
-                _SOCK= pysocket.PySocket()
+                _SOCK= pysocket.PySocket(fingerprints=Fingerprints(),
+                        ca_certs=_CA_CERTS)
             if not _SOCK.connected:
                 _SOCK.connect(host, port)
             args = (None, None, None, 'common', 'db', method) + args
             logging.getLogger('rpc.request').info(repr(args))
-            _SOCK.send(args)
+            try:
+                _SOCK.send(args)
+            except Exception, exception:
+                if exception[0] == 32:
+                    _SOCK.reconnect()
+                    _SOCK.send(args)
+                else:
+                    raise
             res = _SOCK.receive()
             SECURE = _SOCK.ssl
         finally:
@@ -78,7 +91,8 @@ def server_version(host, port):
             if _SOCK and (_SOCK.hostname != host or _SOCK.port != port):
                 _SOCK.disconnect()
             if _SOCK is None:
-                _SOCK = pysocket.PySocket()
+                _SOCK = pysocket.PySocket(fingerprints=Fingerprints(),
+                        ca_certs=_CA_CERTS)
             if not _SOCK.connected:
                 _SOCK.connect(host, port)
             args = (None, None, None, 'common', None, 'version')
@@ -111,7 +125,8 @@ def login(username, password, host, port, database):
             if _SOCK and (_SOCK.hostname != host or _SOCK.port != port):
                 _SOCK.disconnect()
             if _SOCK is None:
-                _SOCK = pysocket.PySocket()
+                _SOCK = pysocket.PySocket(fingerprints=Fingerprints(),
+                        ca_certs=_CA_CERTS)
             if not _SOCK.connected:
                 _SOCK.connect(host, port)
             args = (database, username, password, 'common', 'db', 'login')
@@ -168,9 +183,8 @@ def logout():
 
 def context_reload():
     global CONTEXT, TIMEZONE
-    user = RPCProxy('res.user')
     try:
-        context = user.get_preferences(True, {})
+        context = execute('model', 'res.user', 'get_preferences', True, {})
     except:
         return
     CONTEXT = {}
@@ -224,23 +238,3 @@ def execute(*args):
 
 def execute_nonblocking(*args):
     return _execute(False, *args)
-
-class RPCProxy(object):
-
-    def __init__(self, name):
-        self.name = name
-        self.__attrs = {}
-
-    def __getattr__(self, attr):
-        if attr not in self.__attrs:
-            self.__attrs[attr] = RPCFunction(self.name, attr)
-        return self.__attrs[attr]
-
-class RPCFunction(object):
-
-    def __init__(self, name, func_name):
-        self.name = name
-        self.func = func_name
-
-    def __call__(self, *args):
-        return execute('model', self.name, self.func, *args)
